var Generator = require('yeoman-generator');

module.exports = class extends Generator {

  constructor(args, opts) {
    // Calling the super constructor is important so our generator is correctly set up
    super(args, opts);

    this.argument('path');
  }

  prompting() {

    return this.prompt([{
      type    : 'input',
      name    : 'projectName',
      message : 'Your project name',
      default : 'event-test-api'
    },{
      type    : 'input',
      name    : 'containerName',
      message : 'Your container name',
      default : 'event_test'
    },{
      type    : 'input',
      name    : 'postfixPort',
      message : 'Your postfix port',
      default : '88'
    },{
      type    : 'input',
      name    : 'dbName',
      message : 'Your database name',
      default : 'database'
    },{
      type    : 'input',
      name    : 'baseUrl',
      message : 'Your base url',
      default : 'http://event-test-api.dev'
    },{
      type    : 'input',
      name    : 'domain',
      message : 'Your domain',
      default : 'event-test-api.dev'
    }]).then((answers) => {
      this['postfixPort']   = answers.postfixPort;
      this['dbName']        = answers.dbName;
      this['baseUrl']       = answers.baseUrl;
      this['domain']        = answers.domain;
      this['projectName']   = answers.projectName;
      this['containerName'] = answers.containerName;
    });
  }

  getCode() {
    console.log('-- pull template from git --');
    var nodegit = require("nodegit");
    var Clone = nodegit.Clone;

    var done = this.async();

    Clone.clone("https://github.com/akkapong/event-sourcing-template.git", this.options['path']+"/"+this.projectName, null)
      .then(function(repo) {
            done();
            return repo.getMasterCommit();
        },
        function(err){
            done();
            console.log(err);
        }
    );
  }

  writing() {
    
    console.log('-- Create Docker Config file (.env) --');
    this.fs.delete(this.options.path+'/'+this.projectName+'/.docker.env');
    this.fs.copyTpl(
      this.templatePath('.docker.env'),
      this.destinationPath(this.options.path+'/'+this.projectName+'/.docker.env'),
      { 
        dbName: this.dbName,
        baseUrl: this.baseUrl,
        postfixPort: this.postfixPort
     }
    );
    console.log('-- Create Docker compose file --');
    this.fs.delete(this.options.path+'/'+this.projectName+'/docker/docker-compose.yaml');
    this.fs.copyTpl(
      this.templatePath('docker-compose.yaml'),
      this.destinationPath(this.options.path+'/'+this.projectName+'/docker/docker-compose.yaml'),
      { 
        postfixPort: this.postfixPort
        containerName: this.containerName
     }
    );
    console.log('-- Create nginx config --');
    this.fs.delete(this.options.path+'/'+this.projectName+'/docker/nginx-sites/default');
    this.fs.copyTpl(
      this.templatePath('default'),
      this.destinationPath(this.options.path+'/'+this.projectName+'/docker/nginx-sites/default'),
      { 
        domain: this.domain
     }
    );

    console.log('-- Create start server script --');
    this.fs.delete(this.options.path+'/'+this.projectName+'/docker/start_server.sh');
    this.fs.copyTpl(
      this.templatePath('start_server.sh'),
      this.destinationPath(this.options.path+'/'+this.projectName+'/docker/start_server.sh'),
      { 
        containerName: this.containerName
     }
    );

    //remove folder .git
    var cb     = this.async()
    var rimraf = require("rimraf"); 
    var self   = this;

    rimraf(this.options.path+'/'+this.projectName+'/.git', function () {
        cb();
    });

    
  }

  end() {
    console.log("<<<<< INSTALL COMPLETE >>>>");
    console.log("....Start docker....");
    var cmd = require('node-cmd');
    var self = this;
    cmd.run('cd ');
    cmd.get(
        'cd '+this.options.path+'/'+this.projectName+'/docker && sh start_server.sh',
        function(err, data, stderr){
            console.log('Staring docker complete');
            console.log('** Please add '+self.domain+' in your hosts file **');
        }
    );
  }
};